#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gobject
import popen2
import os
import select
import signal
import shutil
import re

from config import *

class TransferMethod (gobject.GObject):
    __gsignals__ = {
        'start-transfer' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        'update' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_FLOAT, )),
        'transfer-finished' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN, ))
        }

    def __init__ (self):
        gobject.GObject.__init__ (self)

    def start_transfer (self):
        self.update (0.0)
        self.emit ('start-transfer')

    def update (self, percent):
        self.emit ('update', percent)

    def transfer_finished (self, success):
        self.emit ('transfer-finished', success)

    def cancel_transfer (self):
        pass
gobject.type_register (TransferMethod)

def timer_cb(fake_method):
    fake_method.frac += 0.1
    fake_method.update(fake_method.frac)
    if fake_method.frac>=1.0:
        fake_method.transfer_finished (True)
        return False
    else:
        return True    
    
class FakeMethod (TransferMethod):
    def do_start_transfer (self):
        self.frac = 0.0
        self.timer_hn = gobject.timeout_add(10, timer_cb, self)

class RsyncMethod (TransferMethod):
    def __init__ (self, src, dest, extra_options = ""):
        self.src = src
        self.dest = dest
        self.extra_options = extra_options
        self.rsync_child = None
        self.update_re = re.compile ('.*, (\d+\.\d)% of \d+\).*', re.S)
        self.update_percent = 0.0
        self.rsync_path = normalize_path ('/usr/bin/rsync')
        TransferMethod.__init__ (self)

    def read_update (self, fd, cb_condition):
        need_update = False

        if cb_condition == gobject.IO_HUP:
            # EOF means our pipe has closed.
            status = self.rsync_child.wait ()
            self.rsync_child = None
            if os.WIFEXITED (status) and os.WEXITSTATUS (status) == 0:
                self.transfer_finished (True)
            else:
                self.transfer_finished (False)
            return False
        

        # We do a quick attempt to get the update percentage
        output = self.rsync_child.fromchild.read(1024)
        m = self.update_re.match (output)
        if m:
            try:
                update_percent = float (m.groups()[0])
            except ValueError:
                pass
            if update_percent > self.update_percent:
                self.update_percent = update_percent
                self.update (self.update_percent)
        return True
        

    def do_start_transfer (self):
        ssh_command = ''
        src = self.src
        dest = self.dest
        if self.src[:6] == 'ssh://':
            ssh_command = '-e ssh -z'
            src = self.src[6:]

        if not src.endswith ('/'): src = src + ('/')
        if not dest.endswith ('/'): dest = dest + ('/')

        rsync_command = '%s %s -a %s --progress %s %s' % (
            self.rsync_path, ssh_command, self.extra_options, src, dest)
        self.rsync_child = popen2.Popen3 (rsync_command, True)
        gobject.io_add_watch (self.rsync_child.fromchild.fileno(),
                              gobject.IO_IN | gobject.IO_HUP,
                              self.read_update)
        self.rsync_child.tochild.close()

    def cancel_transfer (self):
        if self.rsync_child and self.rsync_child.pid >0:
            os.kill (self.rsync_child.pid, signal.SIGKILL)
