#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import LDAPUtils
from config import StatelessConfig

def Usage ():
    print "stateless-status: - Stateless Linux client status\n\nUsage: stateless-status [-u] [-c configuration] [-s snapshot] [-p protocol] <mac-address-list>"

if __name__ == '__main__':
    import sys
    import getopt

    do_update = False
    config_name = None
    snapshot_name = None
    protocol_name = None
    mac_addresses = []

    class BadUsage: pass
    try:
        opts, args = getopt.getopt (sys.argv[1:],
                                    "uc:s:p:l:r:h",
                                    ["update", "configuration", "snapshot", "protocol", "help"])
        for opt, val in opts:
            if opt == '-u' or opt == '--update':
                do_update = True
            
            if opt == '-c' or opt == '--configuration':
                config_name = val
            
            if opt == '-s' or opt == '--snapshot':
                snapshot_name = val
            
            if opt == '-p' or opt == '--protocol':
                protocol_name = val

            if opt == '-h' or opt == '--help':
                Usage ()
                sys.exit (0)

    except (getopt.error, BadUsage):
        Usage ()
        sys.exit (1)

    if len (args) > 0:
        mac_addresses = args

    if do_update and len (mac_addresses) == 0:
        print "No MAC addresses specified for update"
        Usage ()
        sys.exit (1)

    if do_update and config_name == None and snapshot_name == None and protocol_name == None:
        print "No configuration, snapshot or protocol specified for update"
        Usage ()
        sys.exit (1)

    cfg = StatelessConfig ()
    directory = LDAPUtils.ReadOnlyDirectory (cfg)
    srv = LDAPUtils.StatelessService (directory)

    if do_update:
        for mac_address in mac_addresses:
            srv.update_client_status (mac_address, config_name, snapshot_name, protocol_name)
    else:
        if len (mac_addresses) == 0:
            clients = srv.get_client_list ()
            for client in clients:
                mac_addresses.append (client.get_attr_value ("macAddress"))

        if len (mac_addresses) == 0:
            print "No clients"
            sys.exit (0)
        
        print "Clients:"
        for mac_address in mac_addresses:
            print "   %s" % (mac_address)
            
            status = srv.get_client_status (mac_address)
            if status == None:
                print "      (no status)"
                continue

            config_name   = status.get_attr_value ("statelessConfigurationName")
            snapshot_name = status.get_attr_value ("statelessSnapshotName")
            protocol_name = status.get_attr_value ("statelessProtocolName")
            
            if config_name != None:
                print "      Configuration: %s" % (config_name)

            if snapshot_name != None:
                print "      Snapshot: %s" % (snapshot_name)
            
            if protocol_name != None:
                print "      Protocol: %s" % (protocol_name)
            
        sys.exit (0)
